import React, { PropTypes } from 'react'

const notifyResizeStyle = {
    position: 'absolute',
    top: 0,
    left: 0,
    width: '100%',
    height: '100%',
    zIndex: -1,
    overflow: 'hidden',
    display: 'block',
    pointerEvents: 'none',
    opacity: 0,
}

const expandToolStyle = {
  position: 'absolute',
  top: 0,
  left: 0,
  width: '100%',
  height: '100%',
  overflow: 'auto',
}

const contractToolStyle = {
  position: 'absolute',
  top: 0,
  left: 0,
  width: '100%',
  height: '100%',
  overflow: 'auto',
}

const contractToolInnerStyle = {
  position: 'absolute',
  top: 0,
  left: 0,
  width: '200%',
  height: '200%',
}

class NotifyResize extends React.Component {
  constructor(props){
    super(props)
 
    this.state = {
      notifyResizeWidth: 0,
      notifyResizeHeight: 0,

      expandToolWidth: 0,
      expandToolHeight: 0,
      
      contractToolWidth: 0,
      contractToolHeight: 0,
    }

    this.checkResize = this.checkResize.bind(this)
  }

  componentDidMount(){
    this.resetResizeTool()

    // this.refs.contractTool.scrollTop = this.refs.contractTool.offsetHeight
    // this.refs.contractTool.scrollLeft = this.refs.contractTool.offsetWidth
  }

  render() {
    const props = this.props
    const Component = this.props.factory

    const resizeTool = <div 
      ref="notifyResize"
      className="notifyResize"
      style={notifyResizeStyle}
      onScroll={this.checkResize}
    >
      {this.renderExpandTool()}
      {this.renderContractTool()}
    </div>
    
    return <Component 
      {...props} 
      factory={null} 
      resizeTool={resizeTool} 
      ref="component"
    />
  }

  renderExpandTool(){
    return <div 
      ref="expandTool"
      className="expandTool" 
      style={expandToolStyle}
    >
      <div
        ref="expandToolInner"
        className="expandToolInner"
        style={{
          position: 'absolute',
          top: 0,
          left: 0,   
          width: this.state.expandToolWidth,
          height: this.state.expandToolHeight,
        }}
      />
    </div>
  }

  renderContractTool(){
    return <div 
      ref="contractTool" 
      className="contractTool"
      style={contractToolStyle}
      onScroll={this.checkResize}
    >
      <div ref="contractInner" style={contractToolInnerStyle} />
    </div>
  }

  resetResizeTool(){
    // check if they are monted
    if(!this.refs.resizeTool && !this.refs.notifyResize) {
      console.warn(
          'For notifyResize to work you must render resizeTool from {props.resizeTool}'
        )
      return false
    }

    this.setDimensions()
    this.scrollToBottomExpandTool()
  }

  setDimensions(){
    const {
      notifyResizeWidth,
      notifyResizeHeight
    } = this.getDimensions()
    
    // Resize tool will be bigger than it's parent by 1 pixel in each direction
    this.setState({
      notifyResizeWidth,
      notifyResizeHeight,
      expandToolWidth: notifyResizeWidth + 1,
      expandToolHeight: notifyResizeHeight + 1
    })
  }

  getDimensions(){
    const notifyResize = this.refs.notifyResize

    return {    
      notifyResizeWidth: notifyResize.offsetWidth,
      notifyResizeHeight: notifyResize.offsetHeight
    }    
  }

  scrollToBottomExpandTool(){
    // so the scroll moves when element resizes
   
    if (this.refs.notifyResize) {
      setTimeout(() => {
        // scroll to bottom
        this.refs.expandTool.scrollTop = this.refs.expandTool.scrollHeight
        this.refs.expandTool.scrollLeft = this.refs.expandTool.scrollWidth

        this.refs.contractTool.scrollTop = this.refs.contractTool.scrollHeight
        this.refs.contractTool.scrollLeft = this.refs.contractTool.scrollWidth
      }, 0)
    }
  }

  checkResize(){
    const {
      notifyResizeWidth,
      notifyResizeHeight
    } = this.getDimensions()

    if (
      notifyResizeWidth !== this.state.notifyResizeWidth ||
      notifyResizeHeight !== this.state.notifyResizeHeight
      ) {
      // reset resizeToolDimensions
      this.onResize()
      this.resetResizeTool()
    }
  }

  onResize(){
    if (this.props.onResize) {
      this.props.onResize()
    }

    if (this.refs.component.onResize) {
      this.refs.component.onResize()
    }
  }
}

NotifyResize.propTypes = {
  onResize: PropTypes.func
}

const notifyResize = Component => class NotifyResizeWrapper extends React.Component {
  render() {
    return <NotifyResize ref="notifyResizeWrapper" factory={Component} {...this.props} />
  }
}

export default notifyResize

export {
  NotifyResize
}
